import click
import logging
from cli.cli import datalake as datalake_cli
from cli.datalake.datalake import Datalake
from cli.utils import *
from cli.context import Context, pass_context


logger = logging.getLogger()


@datalake_cli.command()
@click.option("--collection", '-c', nargs=1, type=str, help="Collection where to load data")
@click.option("--path", '-p', nargs=1, type=str, help="Path to csv file to be loaded")
@click.option('--example', '-e', is_flag=True, help="Dump template data")
@pass_context
def load(context: Context, collection: str=None, path: str=None, example: bool=None) -> None:
    try:
        if not collection:
            collection = 'default'

        datalake = Datalake(context)
        datalake.load(collection, path, example)

    except Exception as e:
        logger.exception(e)
        click.echo(f"Error loading data: {str(e)}")
        return

@datalake_cli.command()
@click.option("--collection", '-c', nargs=1, type=str, help="Collection where to dump data")
@click.option('--filter', '-f', multiple=True, default=[], help="Filters (should be in the form of key=value)")
@click.option("--path", '-p', nargs=1, type=str, default='.')
@click.option('--example', '-e', is_flag=True, help="Dump template data")
@pass_context
def dump(context: Context, collection: str, path: str, filter: list, example: bool = False) -> None:
    try:
        if not collection:
            collection = 'default'
        datalake = Datalake(context)
        datalake.dump(collection, path, filter, example)
    except Exception as e:
        logger.exception(e)
        click.secho(f"Error dumping data: {str(e)}", fg="red")
        return

@datalake_cli.command()
@pass_context
def list(context: Context) -> None:
    try:
        datalake = Datalake(context)
        collections = datalake.list()
        Printer.print_list(items=collections, header='collection_name')

    except Exception as e:
        click.secho(f"Error listing datalake: {str(e)}", fg="red")
        return


class ComponentDirectoryAlreadyExists(Exception):
   def __init__(self, dir: str):
        super().__init__(f"Directory with name {dir} already exists in path")


class ComponentAlreadyExistsException(Exception):
    def __init__(self, component: str):
        super().__init__(f"Component with name {component} already exists in hub")




from unittest.mock import patch
from cli.cli import component
from cli.tests.test_generic import SplightCLITest
from cli.constants import *
from cli.component.handler import ComponentHandler


class TestConfigure(SplightCLITest):

    def test_configure_requested(self):
        with patch.object(ComponentHandler, "list_components", return_value=[]):
            result = self.runner.invoke(
                component,
                ["list"],
                obj=self.context,
                catch_exceptions=False
            )
        self.assertEqual(result.exit_code, 1)
        self.assertIn("splightcli configure", result.output)

    def test_configure_not_requested(self):
        self.configure()
        with patch.object(ComponentHandler, "list_components", return_value=[]):
            result = self.runner.invoke(
                component,
                ["list"],
                obj=self.context,
                catch_exceptions=False
            )
        self.assertEqual(result.exit_code, 0)
        self.assertIn('NAME', result.output)



import json

from cli.component import run
from cli.component.component import Component
from cli.tests.test_generic import SplightCLITest


class TestRun(SplightCLITest):
    def test_run(self):
        self.component = Component(self.context)
        self.configure()
        result = self.runner.invoke(
            run,
            [self.path, "--input", json.dumps(self.input)],
            obj=self.context,
            catch_exceptions=False,
        )
        self.assertEqual(
            result.output, "Running component...\nHELLO\nHELLO2\n"
        )



import click
import os
import signal
import sys
import logging
import json
from cli.context import pass_context
from cli.cli import component as cli_component
from cli.utils import *
from cli.context import Context
from cli.constants import *
from cli.component.component import Component, ComponentAlreadyExistsException


def signal_handler(sig, frame):
    sys.exit(0)


signal.signal(signal.SIGINT, signal_handler)


logger = logging.getLogger()


@cli_component.command()
@click.argument("name", nargs=1, type=str)
@click.argument("version", nargs=1, type=str)
@pass_context
def create(context: Context, name: str, version: str) -> None:
    try:
        component = Component(context)
        component.create(name, version)
        click.secho(f"Component {name} created successfully", fg="green")

    except Exception as e:
        click.secho(f"Error creating component: {str(e)}", fg="red")
        sys.exit(1)


@cli_component.command()
@click.argument("path", nargs=1, type=str)
@click.option("-f", "--force", is_flag=True, default=False, help="Force the component to be created even if it already exists.")
@pass_context
def push(context: Context, path: str, force: bool) -> None:
    try:
        component = Component(context)
        try:
            component.push(path, force)
            click.secho("Component pushed successfully to Splight Hub", fg="green")
        except ComponentAlreadyExistsException:
            value = click.prompt(click.style(f"This component already exists in Splight Hub (you can use -f to force pushing). Do you want to overwrite it? (y/n)", fg="yellow"), type=str)
            if value in ["y", "Y"]:
                component.push(path, force=True)
                click.secho("Component pushed successfully to Splight Hub", fg="green")
            else:
                click.secho("Component not pushed", fg="blue")

    except Exception as e:
        click.secho(f"Error pushing component: {str(e)}", fg="red")
        sys.exit(1)


@cli_component.command()
@click.argument("name", nargs=1, type=str)
@click.argument("version", nargs=1, type=str)
@pass_context
def pull(context: Context, name: str, version: str) -> None:
    try:
        path = os.path.abspath(".")
        component = Component(context)
        component.pull(name, version)
        click.secho(f"Component {name}-{version} pulled successfully in {path}", fg="green")

    except Exception as e:
        click.secho(f"Error pulling component: {str(e)}", fg="red")
        sys.exit(1)


@cli_component.command()
@click.argument("name", nargs=1, type=str)
@click.argument("version", nargs=1, type=str)
@pass_context
def delete(context: Context, name: str, version: str) -> None:
    try:
        response = click.prompt(click.style(f"Are you sure you want to delete {name}-{version}? This operation will delete the component from Splight Hub and it won't be recoverable. (y/n)", fg="yellow"), type=str, default="n", show_default=False)
        if response not in ["y", "Y"]:
            click.secho("Component not deleted", fg="blue")
            sys.exit(1)
        component = Component(context)
        component.delete(name, version)
        click.secho(f"Component {name}-{version} deleted successfully", fg="green")

    except Exception as e:
        logger.exception(e)
        click.secho(f"Error deleting component: {str(e)}", fg="red")
        sys.exit(1)


@cli_component.command()
@pass_context
def list(context: Context) -> None:
    try:
        results = Component(context).list()
        Printer.print_dict(items=results, headers=['name'])
        return list

    except Exception as e:
        click.secho(f"Error listing component: {str(e)}", fg="red")
        sys.exit(1)


@cli_component.command()
@click.argument("name", nargs=1, type=str)
@pass_context
def versions(context: Context, name: str) -> None:
    try:
        results = Component(context).versions(name)
        Printer.print_dict(items=results, headers=['name', 'version', 'verification', 'privacy_policy'])
        return list

    except Exception as e:
        click.secho(f"Error listing component: {str(e)}", fg="red")
        sys.exit(1)


@cli_component.command()
@click.argument("path", nargs=1, type=str)
@click.option('--input', '-i', help="Input values")
@click.option('--component-id', '-id', help="Component ID")
@pass_context
def run(context: Context, path: str, input: str = None, component_id: str = None) -> None:
    try:
        component = Component(context)
        click.secho("Running component...", fg="green")
        input = json.loads(input) if input else None
        component.run(path, input_parameters=input, component_id=component_id)
    except Exception as e:
        logger.exception(e)
        click.secho(f"Error running component: {str(e)}", fg="red")
        sys.exit(1)


@cli_component.command()
@click.argument("path", nargs=1, type=str)
@pass_context
def install_requirements(context: Context, path: str) -> None:
    try:
        component = Component(context)
        click.secho("Installing component requirements...", fg="green")
        component.install_requirements(path)
    except Exception as e:
        click.secho(f"Error installing component requirements: {str(e)}", fg="red")
        sys.exit(1)



import os
import shutil
from unittest.mock import patch

import requests

from cli.component.component import Component
from cli.component.handler import ComponentHandler
from cli.constants import COMPRESSION_TYPE
from cli.tests.test_generic import SplightCLITest


class TestPull(SplightCLITest):
    def setUp(self):
        super().setUp()
        self.original_component_path = self.path
        self.path = os.path.dirname(os.path.abspath(__file__))
        self.extracted_component_path = os.path.join(
            self.path, f"{self.name}/{self.version}"
        )
        self.component = Component(self.context)

    def test_pull(self):
        with patch.object(
            ComponentHandler, "download_component"
        ) as downloader:
            self.component.pull(self.name, self.version)
            downloader.assert_called_with(self.name, self.version)

    def test_pull_already_exists_in_local(self):
        already_component = os.path.join(
            self.path, f"{self.name}-{self.version}"
        )
        os.mkdir(already_component)
        with self.assertRaises(Exception):
            self.component.pull(self.name, self.version)
        shutil.rmtree(already_component)

    def test_component_download(self):
        try:
            headers = {
                "Authorization": f"Splight {self.context.workspace.settings.SPLIGHT_ACCESS_ID} {self.context.workspace.settings.SPLIGHT_SECRET_KEY}"
            }
            data = {
                "name": self.name,
                "version": self.version,
            }
            with patch.object(
                ComponentHandler, "exists_in_hub", return_value=True
            ):
                compressed_filename = (
                    f"{self.name}-{self.version}.{COMPRESSION_TYPE}"
                )
                file = None
                with open(
                    os.path.join(
                        self.original_component_path, compressed_filename
                    ),
                    "rb",
                ) as f:
                    file = f.read()
                response = requests.Response()
                response.status_code = 200
                response._content = file
                with patch.object(
                    requests, "post", return_value=response
                ) as post:
                    os.chdir(self.path)
                    self.component.pull(self.name, self.version)
                    _, args, kwargs = post.mock_calls[0]
                    self.assertEqual(
                        args[0],
                        f"{self.context.workspace.settings.SPLIGHT_PLATFORM_API_HOST}/hub/download/",
                    )
                    self.assertEqual(
                        kwargs["data"], {**kwargs["data"], **data}
                    )
                    self.assertEqual(
                        kwargs["headers"], {**kwargs["headers"], **headers}
                    )

                    self.assertTrue(
                        os.path.exists(self.extracted_component_path)
                    )
        finally:
            if os.path.exists(self.extracted_component_path):
                shutil.rmtree(self.extracted_component_path)



import click
import logging

from cli.utils.pprint import Printer
from cli.cli import workspace as workspace_cli
from cli.utils import *
from cli.context import Context, pass_context


logger = logging.getLogger()

@workspace_cli.command()
@pass_context
def list(context: Context) -> None:
    try:
        results = context.workspace.list_workspaces()
        results_colors = ['green' if '*' in item else Printer.DEFAULT_COLOR for item in results]
        Printer.print_list(items=results, items_colors=results_colors, header="WORKSPACES")
    except Exception as e:
        click.secho(f"Error configuring Splight Hub: {str(e)}", fg="red")
        return


@workspace_cli.command()
@click.argument("name", nargs=1, type=str)
@pass_context
def create(context: Context, name: str) -> None:
    try:
        context.workspace.create_workspace(name)
        results = context.workspace.list_workspaces()
        results_colors = ['green' if '*' in item else Printer.DEFAULT_COLOR for item in results]
        Printer.print_list(items=results, items_colors=results_colors, header="WORKSPACES")
    except Exception as e:
        click.secho(f"Error configuring Splight Hub: {str(e)}", fg="red")
        return


@workspace_cli.command()
@click.argument("name", nargs=1, type=str)
@pass_context
def delete(context: Context, name: str) -> None:
    try:
        context.workspace.delete_workspace(name)
        click.secho(f"Deleted workspace {name}", fg="green")
    except Exception as e:
        click.secho(e, fg="red")
        return


@workspace_cli.command()
@click.argument("name", nargs=1, type=str)
@pass_context
def select(context: Context, name: str) -> None:
    try:
        context.workspace.select_workspace(name)
        click.secho(f"Current workspace: {name}", fg="green")
    except Exception as e:
        click.secho(f"Error configuring Splight Hub: {str(e)}", fg="red")
        return



import click


class Printer:
    DEFAULT_COLOR = 'white'

    @staticmethod
    def print_dict(items, headers, items_colors=[]):
        format_str = "{:<50}" * len(headers)
        click.secho(format_str.format(*[h.upper() for h in headers]))
        for index, item in enumerate(items):
            try:
                fg = items_colors[index]
            except IndexError:
                fg = Printer.DEFAULT_COLOR
            click.secho(format_str.format(*[item.get(key) for key in headers]))

    @staticmethod
    def print_list(items, header=None, items_colors=[]):
        if header is not None:
            click.secho(header.upper())
        for index, item in enumerate(items):
            try:
                fg = items_colors[index]
            except IndexError:
                fg = Printer.DEFAULT_COLOR
            click.secho(item, fg=fg)
    
    @staticmethod
    def print_value(item, color=None):
        fg = color if color is not None else Printer.DEFAULT_COLOR
        click.secho(item, fg)




import os
from cli.cli import cli
from cli.context import *
from cli.config import config, get_variable, set_variable
from cli.component import *
from cli.database import *
from cli.datalake import *
from cli.deployment import *
from cli.workspace import *
from cli.constants import SPLIGHT_PATH

os.makedirs(SPLIGHT_PATH, exist_ok=True)

__all__ = [
    cli
]



from typing import TypeVar, Callable
from splight_lib import logging
from splight_lib.settings import setup


logger = logging.getLogger()
RetType = TypeVar("RetType")
OriginalFunc = Callable[..., RetType]
DecoratedFunc = Callable[..., RetType]


class MissingConfiguration(Exception):
    pass
    

class FrameworkManager:
    @property
    def setup(self):
        return setup

    def configure(self, environment):
        empty_keys = [key for key, value in environment.items() if value is None]
        if empty_keys:
            raise MissingConfiguration(f"Please run `splightcli configure` and set keys {empty_keys}")
        setup.configure(environment)
